@@ -22,9 +22,9 @@ You should have received a copy of the GNU General Public License
 along with the OCT6100 GPL API; if not, write to the Free Software 
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 
-$Octasic_Release: OCT612xAPI-01.00-PR43 $
+$Octasic_Release: OCT612xAPI-01.00-PR47 $
 
-$Octasic_Revision: 196 $
+$Octasic_Revision: 202 $
 
 \*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
 
@@ -86,6 +86,9 @@ $Octasic_Revision: 196 $
 #define cOCT6100_ERR_NOT_SUPPORTED_RESET_SOUT_ALC_ON_TONES					(0x00028 + cOCT6100_NOT_SUPPORTED_BASE)
 #define cOCT6100_ERR_NOT_SUPPORTED_CHANNEL_ALC_NOISE_BLEED_OUT_TIME			(0x00029 + cOCT6100_NOT_SUPPORTED_BASE)
 
+
+#define cOCT6100_ERR_NOT_SUPPORTED_CHANNEL_ROUT_NOISE_REDUCTION_GAIN		(0x0002B + cOCT6100_NOT_SUPPORTED_BASE)
+
 #define cOCT6100_ERR_NOT_SUPPORTED_BUFFER_PLAYOUT				(0x00100 + cOCT6100_NOT_SUPPORTED_BASE)
 #define cOCT6100_ERR_NOT_SUPPORTED_CHANNEL_CNR					(0x00101 + cOCT6100_NOT_SUPPORTED_BASE)
 #define cOCT6100_ERR_NOT_SUPPORTED_CONF_BRIDGE					(0x00102 + cOCT6100_NOT_SUPPORTED_BASE)
@@ -96,6 +99,7 @@ $Octasic_Revision: 196 $
 #define cOCT6100_ERR_NOT_SUPPORTED_DOMINANT_SPEAKER				(0x00301 + cOCT6100_NOT_SUPPORTED_BASE)
 
 
+
 #define cOCT6100_ERR_OPEN_INVALID_DEVICE						(0x03000 + cOCT6100_ERR_BASE)
 #define cOCT6100_ERR_OPEN_INSUFFICIENT_EXTERNAL_MEMORY			(0x03001 + cOCT6100_ERR_BASE)
 #define cOCT6100_ERR_OPEN_MEMORY_CHIP_SIZE						(0x03002 + cOCT6100_ERR_BASE)
@@ -180,6 +184,10 @@ $Octasic_Revision: 196 $
 #define cOCT6100_ERR_OPEN_PRODUCTION_BIST_CONF_FAILED			(0x03059 + cOCT6100_ERR_BASE)
 #define cOCT6100_ERR_OPEN_PRODUCTION_BIST_POUCH_ERROR			(0x0305A + cOCT6100_ERR_BASE)
 #define cOCT6100_ERR_OPEN_INVALID_TLV_LENGTH					(0x0305B + cOCT6100_ERR_BASE)
+#define cOCT6100_ERR_OPEN_PRODUCTION_BIST_MODE					(0x0305C + cOCT6100_ERR_BASE)
+
+#define cOCT6100_ERR_CAP_PINS_INVALID_CHIP_STATE				(0x0305E + cOCT6100_ERR_BASE)
+#define cOCT6100_ERR_CAP_PINS_INVALID_CAPACITY_VALUE			(0x0305F + cOCT6100_ERR_BASE)
 
 #define cOCT6100_ERR_TSI_CNCT_ALL_CHANNELS_ARE_OPENED			(0x04000 + cOCT6100_ERR_BASE)
 #define cOCT6100_ERR_TSI_CNCT_DISABLED							(0x04001 + cOCT6100_ERR_BASE)
@@ -409,6 +417,7 @@ $Octasic_Revision: 196 $
 #define cOCT6100_ERR_CHANNEL_AUTO_LEVEL_CONTROL_REQUIRED		(0x070C8 + cOCT6100_ERR_BASE)
 
 #define cOCT6100_ERR_CHANNEL_COMFORT_NOISE_REQUIRED				(0x070CB + cOCT6100_ERR_BASE)
+#define cOCT6100_ERR_CHANNEL_ROUT_NOISE_REDUCTION_GAIN			(0x070CC + cOCT6100_ERR_BASE)
 
 #define cOCT6100_ERR_PHASING_TSST_ALL_ENTRIES_ARE_OPENED		(0x08000 + cOCT6100_ERR_BASE)
 #define cOCT6100_ERR_PHASING_TSST_DISABLED						(0x08001 + cOCT6100_ERR_BASE)
