@@ -22,9 +22,9 @@ You should have received a copy of the GNU General Public License
 along with the OCT6100 GPL API; if not, write to the Free Software 
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 
-$Octasic_Release: OCT612xAPI-01.00-PR43 $
+$Octasic_Release: OCT612xAPI-01.00-PR47 $
 
-$Octasic_Revision: 87 $
+$Octasic_Revision: 88 $
 
 \*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
 
@@ -209,6 +209,7 @@ UINT32 Oct6100ChipGetImageInfoDef(
 	f_pChipImageInfo->fPerChannelTailLength			= FALSE;
 	f_pChipImageInfo->fListenerEnhancement			= FALSE;
 	f_pChipImageInfo->fRoutNoiseReduction			= FALSE;
+	f_pChipImageInfo->fRoutNoiseReductionLevel		= FALSE;
 	f_pChipImageInfo->fAnrSnrEnhancement			= FALSE;
 	f_pChipImageInfo->fAnrVoiceNoiseSegregation		= FALSE;
 	f_pChipImageInfo->fToneDisablerVqeActivationDelay = FALSE;
@@ -267,6 +268,7 @@ UINT32 Oct6100ChipGetImageInfo(
 	f_pChipImageInfo->fPerChannelTailDisplacement	= pImageInfo->fPerChannelTailDisplacement;
 	f_pChipImageInfo->fListenerEnhancement			= pImageInfo->fListenerEnhancement;
 	f_pChipImageInfo->fRoutNoiseReduction			= pImageInfo->fRoutNoiseReduction;
+	f_pChipImageInfo->fRoutNoiseReductionLevel		= pImageInfo->fRoutNoiseReductionLevel;
 	f_pChipImageInfo->fAnrSnrEnhancement			= pImageInfo->fAnrSnrEnhancement;
 	f_pChipImageInfo->fAnrVoiceNoiseSegregation		= pImageInfo->fAnrVoiceNoiseSegregation;
 	f_pChipImageInfo->fMusicProtection				= pImageInfo->fMusicProtection;
