@@ -1,4 +1,3 @@
-#ifndef cOCT6100_REMOVE_EVENTS
 /*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*\
 
 File: oct6100_events.c
@@ -23,9 +22,9 @@ You should have received a copy of the GNU General Public License
 along with the OCT6100 GPL API; if not, write to the Free Software 
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 
-$Octasic_Release: OCT612xAPI-01.00-PR43 $
+$Octasic_Release: OCT612xAPI-01.00-PR47 $
 
-$Octasic_Revision: 79 $
+$Octasic_Revision: 80 $
 
 \*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
 
@@ -1343,4 +1342,4 @@ UINT32 Oct6100BufferPlayoutCheckForSpecificEvent(
 }
 #endif
 
-#endif /* cOCT6100_REMOVE_EVENTS */
+
