@@ -22,7 +22,7 @@ You should have received a copy of the GNU General Public License
 along with the OCT6100 GPL API; if not, write to the Free Software 
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 
-$Octasic_Release: OCT612xAPI-01.00-PR43 $
+$Octasic_Release: OCT612xAPI-01.00-PR47 $
 
 $Octasic_Revision: 35 $
 
@@ -380,15 +380,15 @@ f_pszLastCharPtr		Last character in the source string.
 
 \*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
 #if !SKIP_Oct6100ApiStrStr
-PUINT8 Oct6100ApiStrStr( 
-				IN		PUINT8				f_pszSource, 
-				IN		PUINT8				f_pszString, 
-				IN		PUINT8				f_pszLastCharPtr )
+unsigned char const *Oct6100ApiStrStr( 
+				IN		unsigned char const		*f_pszSource, 
+				IN		unsigned char const		*f_pszString, 
+				IN		unsigned char const		*f_pszLastCharPtr )
 {
 	UINT32	ulCurrentPos;
 	UINT32	ulStringLength;
 	UINT32	ulNumMatchingCharFound = 0;
-	PUINT8	pchFirstChar = NULL;
+	unsigned char const	*pchFirstChar = NULL;
 	UINT32	ulSourceLength;
 
 	if ( f_pszLastCharPtr < f_pszSource )
@@ -453,7 +453,7 @@ f_pszString				Source string to count length of.
 \*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
 #if !SKIP_Oct6100ApiStrLen
 UINT32 Oct6100ApiStrLen( 
-				IN		PUINT8		f_pszString )
+				IN		unsigned char const	*f_pszString )
 {
 	UINT32	ulCount = 0;
 
