@@ -23,9 +23,9 @@ You should have received a copy of the GNU General Public License
 along with the OCT6100 GPL API; if not, write to the Free Software 
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 
-$Octasic_Release: OCT612xAPI-01.00-PR43 $
+$Octasic_Release: OCT612xAPI-01.00-PR47 $
 
-$Octasic_Revision: 49 $
+$Octasic_Revision: 50 $
 
 \*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
 
@@ -1014,8 +1014,23 @@ UINT32 Oct6100ApiIsSSTone(
 
 	switch( f_ulToneEventNumber )
 	{
-	case cOCT6100_TONE_SIN_SYSTEM7_2000:
-	case cOCT6100_TONE_SIN_SYSTEM7_1780:
+	case cOCT6100_TONE_SIN_SYSTEM7_2000				:
+	case cOCT6100_TONE_SIN_SYSTEM7_1780				:	
+	case cOCT6100_TONE_ROUT_G168_2100GB_ON			:
+	case cOCT6100_TONE_ROUT_G168_2100GB_WSPR		:	
+	case cOCT6100_TONE_ROUT_G168_1100GB_ON			:	
+	case cOCT6100_TONE_ROUT_G168_2100GB_ON_WIDE_A	:	
+	case cOCT6100_TONE_ROUT_G168_2100GB_ON_WIDE_B	:
+	case cOCT6100_TONE_ROUT_G168_2100GB_WSPR_WIDE	:	
+	case cOCT6100_TONE_SOUT_G168_2100GB_ON			:	
+	case cOCT6100_TONE_SOUT_G168_2100GB_WSPR		:	
+	case cOCT6100_TONE_SOUT_G168_1100GB_ON			:	
+	case cOCT6100_TONE_SOUT_G168_2100GB_ON_WIDE_A	:	
+	case cOCT6100_TONE_SOUT_G168_2100GB_ON_WIDE_B	:	
+	case cOCT6100_TONE_SOUT_G168_2100GB_WSPR_WIDE	:	
+	case cOCT6100_TONE_SIN_SYSTEM5_2400				:
+	case cOCT6100_TONE_SIN_SYSTEM5_2600				:
+	case cOCT6100_TONE_SIN_SYSTEM5_2400_2600		:	
 		*f_fSSTone = TRUE;
 		break;
 	default:
