@@ -3,8 +3,6 @@
 #include <string.h>
 #include <errno.h>
 #include <stdio.h>
-#include <linux/types.h>
-#include <linux/ppp_defs.h> 
 #include <sys/ioctl.h>
 #include <unistd.h>
 #include <stdlib.h>
