@@ -2,7 +2,7 @@
  * PCI RADIO Card  Zapata Telephony PCI Quad Radio Interface driver
  *
  * Written by Jim Dixon <jim@lambdatel.com>
- * Based on previous work by Mark Spencer <markster@linux-support.net>
+ * Based on previous work by Mark Spencer <markster@digium.com>
  * Based on previous works, designs, and archetectures conceived and
  * written by Jim Dixon <jim@lambdatel.com>.
  *
@@ -169,8 +169,8 @@ struct pciradio {
 	volatile unsigned long ioaddr;
 	dma_addr_t 	readdma;
 	dma_addr_t	writedma;
-	volatile int *writechunk;	/* Double-word aligned write memory */
-	volatile int *readchunk;	/* Double-word aligned read memory */
+	volatile unsigned int *writechunk;	/* Double-word aligned write memory */
+	volatile unsigned int *readchunk;	/* Double-word aligned read memory */
 	unsigned char saudio_status[NUM_CHANS];
 	char gotcor[NUM_CHANS];
 	char gotct[NUM_CHANS];
@@ -539,7 +539,7 @@ unsigned long flags;
 static void _do_encdec(struct pciradio *rad)
 {
 int	i,n;
-unsigned char byte1,byte2;
+unsigned char byte1 = 0, byte2 = 0;
 
 	/* return doing nothing if busy */
 	if ((rad->encdec.lastcmd + 2) > jiffies) return;
@@ -712,11 +712,7 @@ static void pciradio_stop_dma(struct pciradio *rad);
 static void pciradio_reset_serial(struct pciradio *rad);
 static void pciradio_restart_dma(struct pciradio *rad);
 
-#ifdef LINUX26
-static irqreturn_t pciradio_interrupt(int irq, void *dev_id, struct pt_regs *regs)
-#else
-static void pciradio_interrupt(int irq, void *dev_id, struct pt_regs *regs)
-#endif
+ZAP_IRQ_HANDLER(pciradio_interrupt)
 {
 	struct pciradio *rad = dev_id;
 	unsigned char ints,byte1,byte2,gotcor,gotctcss,gotslowctcss,ctcss;
@@ -1264,7 +1260,7 @@ static int pciradio_ioctl(struct zt_chan *chan, unsigned int cmd, unsigned long
 				if ((rad->rxindex < stack.p.data) &&
 				  (rad->srxtimer < SRX_TIMEOUT) &&
 				    ((rad->remmode[chan->chanpos - 1] == ZT_RADPAR_REM_SERIAL) ||
-					(!strchr(rad->rxbuf,'\r'))))
+					(!strchr((char *) rad->rxbuf,'\r'))))
 				{
 					spin_unlock_irqrestore(&rad->lock,flags);
 					interruptible_sleep_on_timeout(&mywait,2);
@@ -1666,7 +1662,7 @@ static int __devinit pciradio_init_one(struct pci_dev *pdev, const struct pci_de
 
 			/* Allocate enough memory for two zt chunks, receive and transmit.  Each sample uses
 			   32 bits.  Allocate an extra set just for control too */
-			rad->writechunk = (int *)pci_alloc_consistent(pdev, ZT_MAX_CHUNKSIZE * 2 * 2 * 2 * 4, &rad->writedma);
+			rad->writechunk = pci_alloc_consistent(pdev, ZT_MAX_CHUNKSIZE * 2 * 2 * 2 * 4, &rad->writedma);
 			if (!rad->writechunk) {
 				printk("pciradio: Unable to allocate DMA-able memory\n");
 				if (rad->freeregion)
