@@ -1,7 +1,7 @@
 /*
  * Tormenta 2  Quad-T1 PCI Driver
  *
- * Written by Mark Spencer <markster@linux-support.net>
+ * Written by Mark Spencer <markster@digium.com>
  * Based on previous works, designs, and archetectures conceived and
  * written by Jim Dixon <jim@lambdatel.com>.
  *
@@ -23,8 +23,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
- *
- * $Id: tor2.c 837 2005-11-29 18:42:08Z kpfleming $
  */
 
 #include <linux/kernel.h>
@@ -190,12 +188,7 @@ static int tor2_shutdown(struct zt_span *span);
 static int tor2_rbsbits(struct zt_chan *chan, int bits);
 static int tor2_maint(struct zt_span *span, int cmd);
 static int tor2_ioctl(struct zt_chan *chan, unsigned int cmd, unsigned long data);
-#ifdef LINUX26
-static irqreturn_t tor2_intr(int irq, void *dev_id, struct pt_regs *regs);
-#else
-static void tor2_intr(int irq, void *dev_id, struct pt_regs *regs);
-#endif
-
+ZAP_IRQ_HANDLER(tor2_intr);
 
 /* translations of data channels for 24 channels in a 32 bit PCM highway */
 unsigned datxlt_t1[] = { 
@@ -1188,11 +1181,7 @@ found:
 	return 0;
 }
 
-#ifdef LINUX26
-static irqreturn_t tor2_intr(int irq, void *dev_id, struct pt_regs *regs)
-#else
-static void tor2_intr(int irq, void *dev_id, struct pt_regs *regs)
-#endif
+ZAP_IRQ_HANDLER(tor2_intr)
 {
 	int n, i, j, k, syncsrc;
 	unsigned int rxword,txword;
