@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
  *
- * Modified from original tor.c by Mark Spencer <markster@linux-support.net>
+ * Modified from original tor.c by Mark Spencer <markster@digium.com>
  *                     original by Jim Dixon <jim@lambdatel.com>
  */
 
@@ -744,11 +744,7 @@ static void torisa_tasklet(unsigned long data)
 
 static int txerrors;
 
-#ifdef LINUX26
-static irqreturn_t torisa_intr(int irq, void *dev_id, struct pt_regs *regs)
-#else
-static void torisa_intr(int irq, void *dev_id, struct pt_regs *regs)
-#endif
+ZAP_IRQ_HANDLER(torisa_intr)
 {
 	static unsigned int passno = 0, mysynccnt = 0, lastsyncsrc = -1;
 	int n, n1, i, j, k, x, mysyncsrc, oldn;
@@ -1143,6 +1139,7 @@ static int __init tor_init(void)
 }
 
 
+#if !defined(LINUX26)
 static int __init set_tor_base(char *str)
 {
 	base = simple_strtol(str, NULL, 0);
@@ -1150,6 +1147,7 @@ static int __init set_tor_base(char *str)
 }
 
 __setup("tor=", set_tor_base);
+#endif
 
 static void __exit tor_exit(void)
 {
@@ -1161,7 +1159,7 @@ static void __exit tor_exit(void)
 		zt_unregister(&spans[1]);
 }
 
-MODULE_AUTHOR("Mark Spencer <markster@linux-support.net>");
+MODULE_AUTHOR("Mark Spencer <markster@digium.com>");
 MODULE_DESCRIPTION("Tormenta ISA Zapata Telephony Driver");
 #ifdef MODULE_LICENSE
 MODULE_LICENSE("GPL");
