@@ -1,8 +1,8 @@
 /*
  * Wilcard X100P FXO Interface Driver for Zapata Telephony interface
  *
- * Written by Mark Spencer <markster@linux-support.net>
- *            Matthew Fredrickson <creslin@linux-support.net>
+ * Written by Mark Spencer <markster@digium.com>
+ *            Matthew Fredrickson <creslin@digium.com>
  *
  * Copyright (C) 2001, Linux Support Services, Inc.
  *
@@ -396,11 +396,7 @@ static void wcfxo_tasklet(unsigned long data)
 static void wcfxo_stop_dma(struct wcfxo *wc);
 static void wcfxo_restart_dma(struct wcfxo *wc);
 
-#ifdef LINUX26
-static irqreturn_t wcfxo_interrupt(int irq, void *dev_id, struct pt_regs *regs)
-#else
-static void wcfxo_interrupt(int irq, void *dev_id, struct pt_regs *regs)
-#endif
+ZAP_IRQ_HANDLER(wcfxo_interrupt)
 {
 	struct wcfxo *wc = dev_id;
 	unsigned char ints;
@@ -411,7 +407,6 @@ static void wcfxo_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 #endif
 
 	ints = inb(wc->ioaddr + WC_INTSTAT);
-	outb(ints, wc->ioaddr + WC_INTSTAT);
 
 
 	if (!ints)
@@ -421,6 +416,8 @@ static void wcfxo_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 		return;
 #endif		
 
+	outb(ints, wc->ioaddr + WC_INTSTAT);
+
 	if (ints & 0x0c) {  /* if there is a rx interrupt pending */
 #ifdef ENABLE_TASKLETS
 		wc->ints = ints;
@@ -1032,7 +1029,7 @@ MODULE_PARM(monitor, "i");
 MODULE_PARM(opermode, "i");
 #endif
 MODULE_DESCRIPTION("Wildcard X100P Zaptel Driver");
-MODULE_AUTHOR("Mark Spencer <markster@linux-support.net>");
+MODULE_AUTHOR("Mark Spencer <markster@digium.com>");
 #ifdef MODULE_LICENSE
 MODULE_LICENSE("GPL");
 #endif
