@@ -1,8 +1,8 @@
 /*
  * Wildcard S100U USB FXS Interface Zapata Telephony Driver
  *
- * Written by Mark Spencer <markster@linux-support.net>
- *            Matthew Fredrickson <creslin@linux-support.net>
+ * Written by Mark Spencer <markster@digium.com>
+ *            Matthew Fredrickson <creslin@digium.com>
  *
  * Copyright (C) 2001, Linux Support Services, Inc.
  *
@@ -1514,7 +1514,7 @@ static void __exit wc_cleanup(void)
 	usb_deregister(&wc_usb_driver);
 }
 
-MODULE_AUTHOR("Mark Spencer <markster@linux-support.net>");
+MODULE_AUTHOR("Mark Spencer <markster@digium.com>");
 MODULE_DESCRIPTION("Wildcard USB FXS Interface driver");
 #ifdef MODULE_LICENSE
 MODULE_LICENSE("GPL");
