@@ -1,9 +1,9 @@
 /*
  * Linux Support Services, Inc.  Wildcard T100P T1/PRI card Driver
  *
- * Written by Mark Spencer <markster@linux-support.net>
- *            Matthew Fredrickson <creslin@linux-support.net>
- *            William Meadows <wmeadows@linux-support.net>
+ * Written by Mark Spencer <markster@digium.com>
+ *            Matthew Fredrickson <creslin@digium.com>
+ *            William Meadows <wmeadows@digium.com>
  *
  * Copyright (C) 2001, Linux Support Services, Inc.
  *
@@ -22,8 +22,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
- *
- * $Id: wct1xxp.c 837 2005-11-29 18:42:08Z kpfleming $
  */
 
 #include <linux/kernel.h>
@@ -1126,11 +1124,7 @@ static void __t1xxp_do_counters(struct t1xxp *wc)
 	}
 }
 
-#ifdef LINUX26
-static irqreturn_t t1xxp_interrupt(int irq, void *dev_id, struct pt_regs *regs)
-#else
-static void t1xxp_interrupt(int irq, void *dev_id, struct pt_regs *regs)
-#endif
+ZAP_IRQ_HANDLER(t1xxp_interrupt)
 {
 	struct t1xxp *wc = dev_id;
 	unsigned char ints;
@@ -1138,8 +1132,6 @@ static void t1xxp_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 	int x;
 
 	ints = inb(wc->ioaddr + WC_INTSTAT);
-	outb(ints, wc->ioaddr + WC_INTSTAT);
-
 	if (!ints)
 #ifdef LINUX26
 		return IRQ_NONE;
@@ -1147,6 +1139,8 @@ static void t1xxp_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 		return;
 #endif		
 
+	outb(ints, wc->ioaddr + WC_INTSTAT);
+
 	if (!wc->intcount) {
 		if (debug) printk("Got interrupt: 0x%04x\n", ints);
 	}
@@ -1407,7 +1401,7 @@ module_param(debug, int, 0600);
 MODULE_PARM(debug, "i");
 #endif
 MODULE_DESCRIPTION("Wildcard T100P/E100P Zaptel Driver");
-MODULE_AUTHOR("Mark Spencer <markster@linux-support.net>");
+MODULE_AUTHOR("Mark Spencer <markster@digium.com>");
 #ifdef MODULE_LICENSE
 MODULE_LICENSE("GPL");
 #endif
