@@ -40,13 +40,16 @@
 #endif
 #ifdef LINUX26
 #include <linux/moduleparam.h>
-#ifdef HOTPLUG_FIRMWARE
-#include <linux/firmware.h>
-#endif
 #endif
+
 #include "wct4xxp.h"
 #include "vpm450m.h"
 
+#ifdef HOTPLUG_FIRMWARE
+static const char *oct064_firmware = "OCT6114-64D.ima";
+static const char *oct128_firmware = "OCT6114-128D.ima";
+#endif
+
 /*
  * Tasklets provide better system interactive response at the cost of the
  * possibility of losing a frame of data at very infrequent intervals.  If
@@ -61,7 +64,7 @@
 
 
 /* Work queues are a way to better distribute load on SMP systems */
-#ifdef LINUX26
+#if defined(LINUX26) && (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,20))
 /*
  * Work queues can significantly improve performance and scalability
  * on multi-processor machines, but requires bypassing some kernel
@@ -396,11 +399,17 @@ static struct t4 *cards[MAX_T4_CARDS];
 #define MAX_TDM_CHAN 32
 #define MAX_DTMF_DET 16
 
+static inline unsigned int __t4_pci_in(struct t4 *wc, const unsigned int addr)
+{
+	unsigned int res = readl(&wc->membase[addr]);
+	return res;
+}
+
 static inline void __t4_pci_out(struct t4 *wc, const unsigned int addr, const unsigned int value)
 {
 	unsigned int tmp;
-	wc->membase[addr] = cpu_to_le32(value);
-	tmp = le32_to_cpu(wc->membase[WC_VERSION]);
+	writel(value, &wc->membase[addr]);
+	tmp = __t4_pci_in(wc, WC_VERSION);
 	if ((tmp & 0xffff0000) != 0xc01a0000)
 		printk("TE4XXP: Version Synchronization Error!\n");
 #if 0
@@ -411,11 +420,6 @@ static inline void __t4_pci_out(struct t4 *wc, const unsigned int addr, const un
 #endif		
 }
 
-static inline unsigned int __t4_pci_in(struct t4 *wc, const unsigned int addr)
-{
-	return le32_to_cpu(wc->membase[addr]);
-}
-
 static inline void __t4_gpio_set(struct t4 *wc, unsigned bits, unsigned int val)
 {
 	unsigned int newgpio;
@@ -2238,11 +2242,7 @@ static inline void __handle_leds(struct t4 *wc)
 }
 
 #ifdef SUPPORT_GEN1
-#ifdef LINUX26
-static irqreturn_t t4_interrupt(int irq, void *dev_id, struct pt_regs *regs)
-#else
-static void t4_interrupt(int irq, void *dev_id, struct pt_regs *regs)
-#endif
+ZAP_IRQ_HANDLER(t4_interrupt)
 {
 	struct t4 *wc = dev_id;
 	unsigned long flags;
@@ -2261,8 +2261,6 @@ static void t4_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 	inirq = 1;
 	/* Make sure it's really for us */
 	status = t4_pci_in(wc, WC_INTR);
-	t4_pci_out(wc, WC_INTR, 0);
-
 	/* Ignore if it's not for us */
 	if (!status)
 #ifdef LINUX26
@@ -2271,6 +2269,8 @@ static void t4_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 		return;
 #endif		
 
+	t4_pci_out(wc, WC_INTR, 0);
+
 	if (!wc->spansstarted) {
 		printk("Not prepped yet!\n");
 #ifdef LINUX26
@@ -2395,11 +2395,7 @@ static inline void __t4_framer_interrupt(struct t4 *wc, int span)
 	}
 }
 
-#ifdef LINUX26
-static irqreturn_t t4_interrupt_gen2(int irq, void *dev_id, struct pt_regs *regs)
-#else
-static void t4_interrupt_gen2(int irq, void *dev_id, struct pt_regs *regs)
-#endif
+ZAP_IRQ_HANDLER(t4_interrupt_gen2)
 {
 	struct t4 *wc = dev_id;
 	unsigned long flags;
@@ -2420,9 +2416,6 @@ static void t4_interrupt_gen2(int irq, void *dev_id, struct pt_regs *regs)
 	inirq = 1;
 	/* Make sure it's really for us */
 	status = t4_pci_in(wc, WC_INTR);
-#if 1
-	t4_pci_out(wc, WC_INTR, status & 0x00000008);
-#endif
 
 	/* Ignore if it's not for us */
 	if (!(status & 0x7)) {
@@ -2432,6 +2425,11 @@ static void t4_interrupt_gen2(int irq, void *dev_id, struct pt_regs *regs)
 		return;
 #endif		
 	}
+
+#ifdef ENABLE_WORKQUEUES
+	t4_pci_out(wc, WC_INTR, status & 0x00000008);
+#endif
+
 	if (!wc->spansstarted) {
 		printk("Not prepped yet!\n");
 #ifdef LINUX26
@@ -2677,7 +2675,15 @@ static void t4_vpm450_init(struct t4 *wc)
 	unsigned int check1, check2;
 	int laws[4] = { 0, };
 	int x;
-	int passedinit = 0;
+	unsigned int vpm_capacity;
+	struct firmware embedded_firmware;
+	const struct firmware *firmware = &embedded_firmware;
+#if !defined(HOTPLUG_FIRMWARE)
+	extern void _binary_OCT6114_64D_ima_size;
+	extern void _binary_OCT6114_128D_ima_size;
+	extern u8 _binary_OCT6114_64D_ima_start[];
+	extern u8 _binary_OCT6114_128D_ima_start[];
+#endif
 
 	if (!vpmsupport) {
 		printk("VPM450: Support Disabled\n");
@@ -2703,53 +2709,65 @@ static void t4_vpm450_init(struct t4 *wc)
 			laws[x] = 1;
 	}
 
-#ifndef HOTPLUG_FIRMWARE
-	if ((wc->vpm450m = init_vpm450m(wc, laws, wc->numspans)))
-		passedinit = 1;
+	switch ((vpm_capacity = get_vpm450m_capacity(wc))) {
+	case 64:
+#if defined(HOTPLUG_FIRMWARE)
+		if ((request_firmware(&firmware, oct064_firmware, &wc->dev->dev) != 0) ||
+		    !firmware) {
+			printk("VPM450: firmware %s not available from userspace\n", oct064_firmware);
+			return;
+		}
 #else
-	{
-		const struct firmware *firmware;
-		
-		/* if this is a 4-span card, just try using the 128-channel firmware;
-		   otherwise, try both, because a 2-span card could potentially have
-		   a 128-channel Octasic module
+		embedded_firmware.data = _binary_OCT6114_64D_ima_start;
+		/* Yes... this is weird. objcopy gives us a symbol containing
+		   the size of the firmware, not a pointer to a variable containing
+		   the size. The only way we can get the value of the symbol
+		   is to take its address, so we define it as a pointer and
+		   then cast that value to the proper type.
 		*/
-		if (wc->numspans < 3) {
-			if ((request_firmware(&firmware, oct064_firmware, &wc->dev->dev) != 0) ||
-			    !firmware) {
-				printk("VPM450: firmware %s not available from userspace\n", oct064_firmware);
-				return;
-			} else {
-				if ((wc->vpm450m = init_vpm450m(wc, laws, wc->numspans, firmware)))
-					passedinit=1;
-
-				release_firmware(firmware);
-			}
-		}
-
-		if (!passedinit) {
-			if ((request_firmware(&firmware, oct128_firmware, &wc->dev->dev) != 0) ||
-			    !firmware) {
-				printk("VPM450: firmware %s not available from userspace\n", oct128_firmware);
-				return;
-			} else {
-				if ((wc->vpm450m = init_vpm450m(wc, laws, wc->numspans, firmware)))
-					passedinit=1;
-
-				release_firmware(firmware);
-			}
-		}
-	}
+		embedded_firmware.size = (size_t) &_binary_OCT6114_64D_ima_size;
 #endif
-
-	if (!passedinit) {
-		printk("VPM450: Failed to initialize\n");
+		break;
+	case 128:
+#if defined(HOTPLUG_FIRMWARE)
+		if ((request_firmware(&firmware, oct128_firmware, &wc->dev->dev) != 0) ||
+		    !firmware) {
+			printk("VPM450: firmware %s not available from userspace\n", oct128_firmware);
+			return;
+		}
+#else
+		embedded_firmware.data = _binary_OCT6114_128D_ima_start;
+		/* Yes... this is weird. objcopy gives us a symbol containing
+		   the size of the firmware, not a pointer to a variable containing
+		   the size. The only way we can get the value of the symbol
+		   is to take its address, so we define it as a pointer and
+		   then cast that value to the proper type.
+		*/
+		embedded_firmware.size = (size_t) &_binary_OCT6114_128D_ima_size;
+#endif
+		break;
+	default:
+		printk("Unsupported channel capacity found on VPM module (%d).\n", vpm_capacity);
 		return;
 	}
+
+	if (!(wc->vpm450m = init_vpm450m(wc, laws, wc->numspans, firmware))) {
+		printk("VPM450: Failed to initialize\n");
+#if defined(HOTPLUG_FIRMWARE)
+		release_firmware(firmware);
+#endif
+		return;
+	}
+
+#if defined(HOTPLUG_FIRMWARE)
+	release_firmware(firmware);
+#endif
+
 	if (vpmdtmfsupport == -1) {
 		printk("VPM450: hardware DTMF disabled.\n");
 		vpmdtmfsupport = 0;
 	}
+
 	wc->vpm = T4_VPM_PRESENT;
 	printk("VPM450: Present and operational servicing %d span(s)\n", wc->numspans);
 }
@@ -3380,6 +3398,9 @@ static void __exit t4_cleanup(void)
 
 MODULE_AUTHOR("Mark Spencer");
 MODULE_DESCRIPTION("Unified TE4XXP-TE2XXP PCI Driver");
+#if defined(MODULE_ALIAS)
+MODULE_ALIAS("wct2xxp");
+#endif
 #ifdef MODULE_LICENSE
 MODULE_LICENSE("GPL");
 #endif
