@@ -1,8 +1,8 @@
 /*
  * Wilcard TDM400P TDM FXS/FXO Interface Driver for Zapata Telephony interface
  *
- * Written by Mark Spencer <markster@linux-support.net>
- *            Matthew Fredrickson <creslin@linux-support.net>
+ * Written by Mark Spencer <markster@digium.com>
+ *            Matthew Fredrickson <creslin@digium.com>
  *
  * Copyright (C) 2001, Linux Support Services, Inc.
  *
@@ -345,8 +345,8 @@ struct wctdm {
 	unsigned long ioaddr;
 	dma_addr_t 	readdma;
 	dma_addr_t	writedma;
-	volatile int *writechunk;					/* Double-word aligned write memory */
-	volatile int *readchunk;					/* Double-word aligned read memory */
+	volatile unsigned int *writechunk;				/* Double-word aligned write memory */
+	volatile unsigned int *readchunk;				/* Double-word aligned read memory */
 	struct zt_chan chans[NUM_CARDS];
 };
 
@@ -1024,11 +1024,7 @@ static inline void wctdm_proslic_check_hook(struct wctdm *wc, int card)
 	wc->mod[card].fxs.lastrxhook = hook;
 }
 
-#ifdef LINUX26
-static irqreturn_t wctdm_interrupt(int irq, void *dev_id, struct pt_regs *regs)
-#else
-static void wctdm_interrupt(int irq, void *dev_id, struct pt_regs *regs)
-#endif
+ZAP_IRQ_HANDLER(wctdm_interrupt)
 {
 	struct wctdm *wc = dev_id;
 	unsigned char ints;
@@ -1036,7 +1032,6 @@ static void wctdm_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 	int mode;
 
 	ints = inb(wc->ioaddr + WC_INTSTAT);
-	outb(ints, wc->ioaddr + WC_INTSTAT);
 
 	if (!ints)
 #ifdef LINUX26
@@ -1045,6 +1040,8 @@ static void wctdm_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 		return;
 #endif		
 
+	outb(ints, wc->ioaddr + WC_INTSTAT);
+
 	if (ints & 0x10) {
 		/* Stop DMA, wait for watchdog */
 		printk("TDM PCI Master abort\n");
@@ -2208,7 +2205,7 @@ static int __devinit wctdm_init_one(struct pci_dev *pdev, const struct pci_devic
 
 			/* Allocate enough memory for two zt chunks, receive and transmit.  Each sample uses
 			   32 bits.  Allocate an extra set just for control too */
-			wc->writechunk = (int *)pci_alloc_consistent(pdev, ZT_MAX_CHUNKSIZE * 2 * 2 * 2 * 4, &wc->writedma);
+			wc->writechunk = pci_alloc_consistent(pdev, ZT_MAX_CHUNKSIZE * 2 * 2 * 2 * 4, &wc->writedma);
 			if (!wc->writechunk) {
 				printk("wctdm: Unable to allocate DMA-able memory\n");
 				if (wc->freeregion)
@@ -2428,6 +2425,9 @@ MODULE_PARM(alawoverride, "i");
 #endif
 MODULE_DESCRIPTION("Wildcard TDM400P Zaptel Driver");
 MODULE_AUTHOR("Mark Spencer <markster@digium.com>");
+#if defined(MODULE_ALIAS)
+MODULE_ALIAS("wcfxs");
+#endif
 #ifdef MODULE_LICENSE
 MODULE_LICENSE("GPL");
 #endif
