@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
  *
- * $Id: wcte11xp.c 1573 2006-11-08 23:04:33Z kpfleming $
+ * $Id: wcte11xp.c 1936 2007-01-23 21:28:14Z mattf $
  */
 
 #include <linux/kernel.h>
@@ -1257,11 +1257,7 @@ static void __t1_do_counters(struct t1 *wc)
 	}
 }
 
-#ifdef LINUX26
-static irqreturn_t t1xxp_interrupt(int irq, void *dev_id, struct pt_regs *regs)
-#else
-static void t1xxp_interrupt(int irq, void *dev_id, struct pt_regs *regs)
-#endif
+ZAP_IRQ_HANDLER(t1xxp_interrupt)
 {
 	struct t1 *wc = dev_id;
 	unsigned char ints;
@@ -1269,8 +1265,6 @@ static void t1xxp_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 	int x;
 
 	ints = inb(wc->ioaddr + WC_INTSTAT);
-	outb(ints, wc->ioaddr + WC_INTSTAT);
-
 	if (!ints)
 #ifdef LINUX26
 		return IRQ_NONE;
@@ -1278,6 +1272,8 @@ static void t1xxp_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 		return;
 #endif		
 
+	outb(ints, wc->ioaddr + WC_INTSTAT);
+
 	if (!wc->intcount) {
 		if (debug) printk("Got interrupt: 0x%04x\n", ints);
 	}
@@ -1381,7 +1377,7 @@ static int t1xxp_hardware_init(struct t1 *wc)
 	}
 
 	/* Check out the controller */
-	if (debug); printk("Controller version: %02x\n", control_get_reg(wc, WC_VERSION));
+	if (debug) printk("Controller version: %02x\n", control_get_reg(wc, WC_VERSION));
 
 
 	control_set_reg(wc, WC_LEDTEST, 0x00);
