@@ -23,22 +23,18 @@
  */
 
 #include "xpd.h"
-#include "slic.h"
 
 enum fxs_opcodes {
 	XPROTO_NAME(FXS, SIG_CHANGED)		= 0x06,
 /**/
-	XPROTO_NAME(FXS, SLIC_WRITE)		= 0x0F,	/* Write to SLIC */
-	XPROTO_NAME(FXS, CHAN_ENABLE)		= 0x0F,	/* Write to SLIC */
+	XPROTO_NAME(FXS, XPD_STATE)		= 0x0F,	/* Write to SLIC */
 	XPROTO_NAME(FXS, CHAN_POWER)		= 0x0F,	/* Write to SLIC */
 	XPROTO_NAME(FXS, CHAN_CID)		= 0x0F,	/* Write to SLIC */
 	XPROTO_NAME(FXS, RING)			= 0x0F,	/* Write to SLIC */
 	XPROTO_NAME(FXS, LED)			= 0x0F,	/* Write to SLIC */
 	XPROTO_NAME(FXS, RELAY_OUT)		= 0x0F,	/* Write to SLIC */
-	XPROTO_NAME(FXS, SLIC_INIT)		= 0x0F,	/* Write to SLIC */
-	XPROTO_NAME(FXS, SLIC_QUERY)		= 0x0F,	/* Write to SLIC */
 /**/
-	XPROTO_NAME(FXS, SLIC_REPLY)		= 0x10,
+	XPROTO_NAME(FXS, REGISTER_REPLY)	= 0x10,
 };
 
 
@@ -47,12 +43,8 @@ DEF_RPACKET_DATA(FXS, SIG_CHANGED,
 	xpp_line_t	sig_status;	/* channels: lsb=1, msb=8 */
 	xpp_line_t	sig_toggles;	/* channels: lsb=1, msb=8 */
 	);
-DEF_RPACKET_DATA(FXS, SLIC_REPLY,	/* Get status of a single SLIC (for debugging) */
-	xpp_line_t	lines;
-	slic_reply_t	info;
-	);
-DEF_RPACKET_DATA(FXS, SLIC_WRITE,
-	slic_cmd_t	slic_cmd;
+DEF_RPACKET_DATA(FXS, REGISTER_REPLY,	/* Get status of a single SLIC (for debugging) */
+	reg_cmd_t	reg_cmd;
 	);
 
 #endif	/* CARD_FXS_H */
