@@ -31,6 +31,9 @@ DEF_RPACKET_DATA(GLOBAL, DEV_DESC,
 	byte		type;		/* LSB: 1 - to_phone, 0 - to_line */
 	xpp_line_t	line_status;	/* hook/ring status, depending on unit */
 	);
+DEF_RPACKET_DATA(GLOBAL, REGISTER_REQUEST,
+	reg_cmd_t	reg_cmd;
+	);
 DEF_RPACKET_DATA(GLOBAL, PCM_WRITE,
 	xpp_line_t	lines;
 	byte		pcm[PCM_CHUNKSIZE];
@@ -45,6 +48,12 @@ DEF_RPACKET_DATA(GLOBAL, SYNC_SOURCE,
 DEF_RPACKET_DATA(GLOBAL, SYNC_REPLY,
 	byte		mask;
 	);
+DEF_RPACKET_DATA(GLOBAL, ERROR_CODE,
+	byte		errorcode;
+	union {
+		reg_cmd_t	bad_spi_cmd;
+	} info;
+	);
 
 
 /* 0x04 */ DECLARE_CMD(GLOBAL, DESC_REQ, int xpd_num);
@@ -52,5 +61,6 @@ DEF_RPACKET_DATA(GLOBAL, SYNC_REPLY,
 /* 0x11 */ DECLARE_CMD(GLOBAL, PCM_WRITE, xpp_line_t lines, volatile byte *buf);
 
 extern xproto_table_t PROTO_TABLE(GLOBAL);
+int run_initialize_registers(xpd_t *xpd);
 
 #endif	/* CARD_GLOBAL_H */
