@@ -8,10 +8,23 @@
 #include <usb.h>
 #include "hexfile.h"
 
-static const char rcsid[] = "$Id: fpga_load.c 1457 2006-09-09 15:24:12Z tzafrir $";
+static const char rcsid[] = "$Id: fpga_load.c 2239 2007-02-27 06:14:18Z tzafrir $";
 
-#define	ERR(fmt, arg...)	fprintf(stderr, "%s: ERROR: " fmt, progname, ## arg)
-#define	INFO(fmt, arg...)	fprintf(stderr, "%s: " fmt, progname, ## arg)
+#define	ERR(fmt, arg...) do { \
+				if(verbose >= LOG_ERR) \
+					fprintf(stderr, "%s: ERROR: " fmt, \
+						progname, ## arg); \
+			} while(0);
+#define	INFO(fmt, arg...) do { \
+				if(verbose >= LOG_INFO) \
+					fprintf(stderr, "%s: " fmt, \
+						progname, ## arg); \
+			} while(0);
+#define	DBG(fmt, arg...) do { \
+				if(verbose >= LOG_DEBUG) \
+					fprintf(stderr, "%s: DBG: " fmt, \
+						progname, ## arg); \
+			} while(0);
 
 static int	verbose = LOG_WARNING;
 static char	*progname;
@@ -72,7 +85,15 @@ enum fpga_load_status {
 	FW_CONFIG_DONE	= 8
 };
 
-int my_usb_device(struct usb_device *dev, usb_dev_handle *handle);
+struct my_usb_device {
+	struct usb_device	*dev;
+	usb_dev_handle		*handle;
+	char			iManufacturer[BUFSIZ];
+	char			iProduct[BUFSIZ];
+	char			iSerialNumber[BUFSIZ];
+	int			is_usb2;
+	struct myeeprom		eeprom;
+};
 
 const char *load_status2str(enum fpga_load_status s)
 {
@@ -85,33 +106,43 @@ const char *load_status2str(enum fpga_load_status s)
 	}
 }
 
-int path_of_dev(char buf[], unsigned int buflen, struct usb_device *dev)
-{
-	return snprintf(buf, buflen, "/proc/bus/usb/%s/%s", dev->bus->dirname, dev->filename);
-}
-
 struct usb_device *dev_of_path(const char *path)
 {
 	struct usb_bus		*bus;
 	struct usb_device	*dev;
 	char			dirname[PATH_MAX];
 	char			filename[PATH_MAX];
-	const char		prefix[] = "/proc/bus/usb/";
-	const int		prefix_len = strlen(prefix);
 	const char		*p;
 	int			bnum;
 	int			dnum;
 	int			ret;
 
 	assert(path != NULL);
-	if(strncmp(prefix, path, prefix_len) != 0) {
-		ERR("wrong path: '%s'\n", path);
+	if(access(path, F_OK) < 0) {
+		perror(path);
 		return NULL;
 	}
-	p = path + prefix_len;
-	ret = sscanf(p, "%d/%d", &bnum, &dnum);
-	if(ret != 2) {
-		ERR("wrong path tail: '%s'\n", p);
+	/* Find last '/' */
+	if((p = memrchr(path, '/', strlen(path))) == NULL) {
+		ERR("Missing a '/' in %s\n", path);
+		return NULL;
+	}
+	/* Get the device number */
+	ret = sscanf(p + 1, "%d", &dnum);
+	if(ret != 1) {
+		ERR("Path tail is not a device number: '%s'\n", p);
+		return NULL;
+	}
+	/* Search for a '/' before that */
+	p = memrchr(path, '/', p - path);
+	if(p == NULL)
+		p = path;		/* Relative path */
+	else
+		p++;			/* skip '/' */
+	/* Get the bus number */
+	ret = sscanf(p, "%d", &bnum);
+	if(ret != 1) {
+		ERR("Path tail is not a bus number: '%s'\n", p);
 		return NULL;
 	}
 	sprintf(dirname, "%03d", bnum);
@@ -149,6 +180,9 @@ int get_usb_string(char *buf, unsigned int len, uint16_t item, usb_dev_handle *h
 #define	MY_EP_OUT	0x04
 #define	MY_EP_IN	0x88
 
+#define	FPGA_EP_OUT	0x02
+#define	FPGA_EP_IN	0x86
+
 #define	TIMEOUT		5000
 
 static const int my_endpoints[MY_ENDPOINTS] = {
@@ -158,34 +192,36 @@ static const int my_endpoints[MY_ENDPOINTS] = {
 	0x88
 };
 
-void usb_cleanup(usb_dev_handle *handle)
+void my_usb_device_cleanup(struct my_usb_device *mydev)
 {
-	if(usb_release_interface(handle, MY_INTERFACE) != 0) {
+	assert(mydev != NULL);
+	if(!mydev->handle) {
+		return;	/* Nothing to do */
+	}
+	if(usb_release_interface(mydev->handle, MY_INTERFACE) != 0) {
 		ERR("Releasing interface: usb: %s\n", usb_strerror());
 	}
-	if(usb_close(handle) != 0) {
+	if(usb_close(mydev->handle) != 0) {
 		ERR("Closing device: usb: %s\n", usb_strerror());
 	}
 }
 
-void print_bcd_ver(const struct myeeprom *eeprom)
+static void show_device_info(const struct my_usb_device *mydev)
 {
-	/* In this case, print only the version. Also note that this 
-	 * is an output, and sent to stdout
-	 */
-	printf("%d.%03d\n", eeprom->release_major, eeprom->release_minor);
-	return;
-}
+	const struct myeeprom	*eeprom;
+	const uint8_t		*data = eeprom->serial;
 
-void dump_eeprom(const struct myeeprom *eeprom)
-{
-	const uint8_t	*data = eeprom->serial;
-
-	INFO("Source:  0x%02X\n", eeprom->source);
-	INFO("Vendor:  0x%04X\n", eeprom->vendor);
-	INFO("Product: 0x%04X\n", eeprom->product);
-	INFO("Release: %d.%03d\n", eeprom->release_major, eeprom->release_minor);
-	INFO("Data:    0x[%02X,%02X,%02X,%02X,%02X,%02X,%02X,%02X]\n",
+	assert(mydev != NULL);
+	eeprom = &mydev->eeprom;
+	data = eeprom->serial;
+	printf("USB    iManufacturer: [%s]\n", mydev->iManufacturer);
+	printf("USB    iProduct:      [%s]\n", mydev->iProduct);
+	printf("USB    iSerialNumber: [%s]\n", mydev->iSerialNumber);
+	printf("EEPROM Source:        0x%02X\n", eeprom->source);
+	printf("EEPROM Vendor:        0x%04X\n", eeprom->vendor);
+	printf("EEPROM Product:       0x%04X\n", eeprom->product);
+	printf("EEPROM Release:       %d.%03d\n", eeprom->release_major, eeprom->release_minor);
+	printf("EEPROM Serial:        0x[%02X,%02X,%02X,%02X,%02X,%02X,%02X,%02X]\n",
 			data[0], data[1], data[2], data[3],
 			data[4], data[5], data[6], data[7]); 
 }
@@ -199,80 +235,80 @@ void dump_packet(const char *buf, int len)
 }
 
 #ifdef	XORCOM_INTERNAL
-int eeprom_set(struct usb_dev_handle *handle, const struct myeeprom *eeprom)
+int eeprom_set(struct my_usb_device *mydev, const struct myeeprom *eeprom)
 {
 	int				ret;
 	int				len;
 	char				buf[PACKET_SIZE];
 	struct fpga_packet_header	*phead = (struct fpga_packet_header *)buf;
 
-	if(verbose >= LOG_INFO)
-		INFO("%s Start...\n", __FUNCTION__);
-	assert(handle != NULL);
+	DBG("%s Start...\n", __FUNCTION__);
+	assert(mydev != NULL);
 	phead->header.op = EEPROM_SET;
 	memcpy(&phead->d.eeprom_set.data, eeprom, EEPROM_SIZE);
 	len = sizeof(phead->d.eeprom_set) + sizeof(phead->header.op);
-	if(verbose >= LOG_INFO) {
-		INFO("%s write %d bytes\n", __FUNCTION__, len);
+	if(verbose >= LOG_DEBUG) {
+		DBG("%s write %d bytes\n", __FUNCTION__, len);
 		dump_packet((char *)phead, len);
 	}
-	ret = usb_bulk_write(handle, MY_EP_OUT, (char *)phead, len, TIMEOUT);
+	ret = usb_bulk_write(mydev->handle, MY_EP_OUT, (char *)phead, len, TIMEOUT);
 	if(ret < 0) {
-		ERR("usb: bulk_write failed (%d)\n", ret);
+		ERR("bulk_write failed: %s\n", usb_strerror());
 		return ret;
 	} else if(ret != len) {
-		ERR("usb: bulk_write short write (%d)\n", ret);
+		ERR("bulk_write short write: %s\n", usb_strerror());
 		return -EFAULT;
 	}
-	ret = usb_bulk_read(handle, MY_EP_IN, buf, sizeof(buf), TIMEOUT);
+	ret = usb_bulk_read(mydev->handle, MY_EP_IN, buf, sizeof(buf), TIMEOUT);
 	if(ret < 0) {
-		ERR("usb: bulk_read failed (%d)\n", ret);
+		ERR("bulk_read failed: %s\n", usb_strerror());
 		return ret;
 	} else if(ret == 0)
 		return 0;
 	phead = (struct fpga_packet_header *)buf;
 	if(phead->header.op == BAD_COMMAND) {
-		ERR("BAD_COMMAND\n");
+		ERR("Firmware rejected EEPROM_SET command\n");
 		return -EINVAL;
 	} else if(phead->header.op != EEPROM_SET) {
 		ERR("Got unexpected reply op=%d\n", phead->header.op);
 		return -EINVAL;
 	}
-	if(verbose >= LOG_INFO) {
-		INFO("%s read %d bytes\n", __FUNCTION__, ret);
+	if(verbose >= LOG_DEBUG) {
+		DBG("%s read %d bytes\n", __FUNCTION__, ret);
 		dump_packet(buf, ret);
 	}
 	return 0;
 }
 #endif
 
-int eeprom_get(struct usb_dev_handle *handle, struct myeeprom *eeprom)
+int eeprom_get(struct my_usb_device *mydev)
 {
 	int				ret;
 	int				len;
 	char				buf[PACKET_SIZE];
 	struct fpga_packet_header	*phead = (struct fpga_packet_header *)buf;
+	struct myeeprom			*eeprom;
 
-	assert(handle != NULL);
-	if(verbose >= LOG_INFO)
-		INFO("%s Start...\n", __FUNCTION__);
+	assert(mydev != NULL);
+	eeprom = &mydev->eeprom;
+	DBG("%s Start...\n", __FUNCTION__);
 	phead->header.op = EEPROM_GET;
 	len = sizeof(phead->header.op);		/* warning: sending small packet */
-	if(verbose >= LOG_INFO) {
-		INFO("%s write %d bytes\n", __FUNCTION__, len);
+	if(verbose >= LOG_DEBUG) {
+		DBG("%s write %d bytes\n", __FUNCTION__, len);
 		dump_packet(buf, len);
 	}
-	ret = usb_bulk_write(handle, MY_EP_OUT, (char *)phead, len, TIMEOUT);
+	ret = usb_bulk_write(mydev->handle, MY_EP_OUT, (char *)phead, len, TIMEOUT);
 	if(ret < 0) {
-		ERR("usb: bulk_write failed (%d)\n", ret);
+		ERR("bulk_write failed: %s\n", usb_strerror());
 		return ret;
 	} else if(ret != len) {
-		ERR("usb: bulk_write short write (%d)\n", ret);
+		ERR("bulk_write short write: %s\n", usb_strerror());
 		return -EFAULT;
 	}
-	ret = usb_bulk_read(handle, MY_EP_IN, buf, sizeof(buf), TIMEOUT);
+	ret = usb_bulk_read(mydev->handle, MY_EP_IN, buf, sizeof(buf), TIMEOUT);
 	if(ret < 0) {
-		ERR("usb: bulk_read failed (%d)\n", ret);
+		ERR("bulk_read failed: %s\n", usb_strerror());
 		return ret;
 	} else if(ret == 0)
 		return 0;
@@ -284,15 +320,15 @@ int eeprom_get(struct usb_dev_handle *handle, struct myeeprom *eeprom)
 		ERR("Got unexpected reply op=%d\n", phead->header.op);
 		return -EINVAL;
 	}
-	if(verbose >= LOG_INFO) {
-		INFO("%s read %d bytes\n", __FUNCTION__, ret);
+	if(verbose >= LOG_DEBUG) {
+		DBG("%s read %d bytes\n", __FUNCTION__, ret);
 		dump_packet(buf, ret);
 	}
 	memcpy(eeprom, &phead->d.eeprom_get.data, EEPROM_SIZE);
 	return 0;
 }
 
-int send_hexline(struct usb_dev_handle *handle, struct hexline *hexline, int seq)
+int send_hexline(struct my_usb_device *mydev, struct hexline *hexline, int seq)
 {
 	int				ret;
 	int				len;
@@ -301,7 +337,7 @@ int send_hexline(struct usb_dev_handle *handle, struct hexline *hexline, int seq
 	struct fpga_packet_header	*phead = (struct fpga_packet_header *)buf;
 	enum fpga_load_status		status;
 
-	assert(handle != NULL);
+	assert(mydev != NULL);
 	assert(hexline != NULL);
 	len = hexline->d.content.header.ll;	/* don't send checksum */
 	data = hexline->d.content.tt_data.data;
@@ -314,26 +350,24 @@ int send_hexline(struct usb_dev_handle *handle, struct hexline *hexline, int seq
 	phead->d.data_packet.reserved = 0x00;
 	memcpy(phead->d.data_packet.data, data, len);
 	len += sizeof(hexline->d.content.header);
-	if(verbose >= LOG_INFO)
-		INFO("%04d+\r", seq);
-	ret = usb_bulk_write(handle, MY_EP_OUT, (char *)phead, len, TIMEOUT);
+	DBG("%04d+\r", seq);
+	ret = usb_bulk_write(mydev->handle, MY_EP_OUT, (char *)phead, len, TIMEOUT);
 	if(ret < 0) {
-		ERR("usb: bulk_write failed (%d)\n", ret);
+		ERR("bulk_write failed: %s\n", usb_strerror());
 		return ret;
 	} else if(ret != len) {
-		ERR("usb: bulk_write short write (%d)\n", ret);
+		ERR("bulk_write short write: %s\n", usb_strerror());
 		return -EFAULT;
 	}
 	if (verbose >= LOG_DEBUG)
 		dump_packet((char*)phead, len);
-	ret = usb_bulk_read(handle, MY_EP_IN, buf, sizeof(buf), TIMEOUT);
+	ret = usb_bulk_read(mydev->handle, MY_EP_IN, buf, sizeof(buf), TIMEOUT);
 	if(ret < 0) {
-		ERR("usb: bulk_read failed (%d)\n", ret);
+		ERR("bulk_read failed: %s\n", usb_strerror());
 		return ret;
 	} else if(ret == 0)
 		return 0;
-	if(verbose >= LOG_INFO)
-		INFO("%04d-\r", seq);
+	DBG("%04d-\r", seq);
 	phead = (struct fpga_packet_header *)buf;
 	if(phead->header.op != STATUS_REPLY) {
 		ERR("Got unexpected reply op=%d\n", phead->header.op);
@@ -347,13 +381,11 @@ int send_hexline(struct usb_dev_handle *handle, struct hexline *hexline, int seq
 		case FW_FAIL_RESET:
 		case FW_FAIL_TRANS:
 			ERR("status reply %s (%d)\n", load_status2str(status), status);
-			if(verbose >= LOG_INFO)
-				dump_packet(buf, ret);
+			dump_packet(buf, ret);
 			return -EPROTO;
 		default:
 			ERR("Unknown status reply %d\n", status);
-			if(verbose >= LOG_INFO)
-				dump_packet(buf, ret);
+			dump_packet(buf, ret);
 			return -EPROTO;
 	}
 	return 0;
@@ -361,7 +393,7 @@ int send_hexline(struct usb_dev_handle *handle, struct hexline *hexline, int seq
 
 //. returns > 0 - ok, the number of lines sent
 //. returns < 0 - error number
-int send_splited_hexline(struct usb_dev_handle *handle, struct hexline *hexline, int seq, uint8_t maxwidth)
+int send_splited_hexline(struct my_usb_device *mydev, struct hexline *hexline, int seq, uint8_t maxwidth)
 {
 	struct hexline *extraline;
 	int linessent = 0;
@@ -370,6 +402,7 @@ int send_splited_hexline(struct usb_dev_handle *handle, struct hexline *hexline,
 	unsigned int this_line = 0;
 	uint8_t bytesleft = 0;
 	
+	assert(mydev != NULL);
 	if(!hexline) {
 		ERR("Bad record %d type = %d\n", seq, hexline->d.content.header.tt);
 		return -EINVAL;
@@ -390,7 +423,7 @@ int send_splited_hexline(struct usb_dev_handle *handle, struct hexline *hexline,
 		extraline->d.content.header.offset	= hexline->d.content.header.offset + extra_offset;
 		extraline->d.content.header.tt		= hexline->d.content.header.tt;
 		memcpy( extraline->d.content.tt_data.data, hexline->d.content.tt_data.data+extra_offset, this_line);
-		status = send_hexline( handle, extraline, seq+linessent );
+		status = send_hexline(mydev, extraline, seq+linessent );
 		// cleanups
 		free(extraline);
 		extra_offset += this_line;
@@ -402,108 +435,131 @@ int send_splited_hexline(struct usb_dev_handle *handle, struct hexline *hexline,
 	return linessent;
 }
 
-int my_usb_device(struct usb_device *dev, usb_dev_handle *handle)
+int my_usb_device_init(const char devpath[], struct my_usb_device *mydev)
 {
 	struct usb_device_descriptor	*dev_desc;
 	struct usb_config_descriptor	*config_desc;
 	struct usb_interface		*interface;
 	struct usb_interface_descriptor	*iface_desc;
 	struct usb_endpoint_descriptor	*endpoint;
-	char				iManufacturer[BUFSIZ];
-	char				iProduct[BUFSIZ];
 	int				ret;
 	int				i;
 
-	assert(dev != NULL);
-	dev_desc = &dev->descriptor;
-	config_desc = dev->config;
+	assert(mydev != NULL);
+	usb_init();
+	usb_find_busses();
+	usb_find_devices();
+	mydev->dev = dev_of_path(devpath);
+	if(!mydev->dev) {
+		ERR("Bailing out\n");
+		return 0;
+	}
+	mydev->handle = usb_open(mydev->dev);
+	if(!mydev->handle) {
+		ERR("Failed to open usb device '%s/%s': %s\n", mydev->dev->bus->dirname, mydev->dev->filename, usb_strerror());
+		return 0;
+	}
+	if(usb_reset(mydev->handle) != 0) {
+		ERR("Reseting device: usb: %s\n", usb_strerror());
+		return 0;
+	}
+	if(usb_set_configuration(mydev->handle, MY_CONFIG) != 0) {
+		ERR("usb: %s\n", usb_strerror());
+		return 0;
+	}
+	if(usb_claim_interface(mydev->handle, MY_INTERFACE) != 0) {
+		ERR("usb: %s\n", usb_strerror());
+		return 0;
+	}
+	dev_desc = &mydev->dev->descriptor;
+	config_desc = mydev->dev->config;
 	interface = config_desc->interface;
 	iface_desc = interface->altsetting;
-	if(verbose >= LOG_INFO)
-		INFO("Vendor:Product=%04X:%04X Class=%d (endpoints=%d)\n",
-				dev_desc->idVendor, dev_desc->idProduct, dev_desc->bDeviceClass, iface_desc->bNumEndpoints);
+	INFO("Vendor:Product=%04X:%04X Class=%d (endpoints=%d)\n",
+			dev_desc->idVendor,
+			dev_desc->idProduct,
+			dev_desc->bDeviceClass,
+			iface_desc->bNumEndpoints);
 	if(iface_desc->bInterfaceClass != 0xFF) {
 		ERR("Wrong Interface class %d\n", iface_desc->bInterfaceClass);
-		return -EINVAL;
+		return 0;
 	}
 	if(iface_desc->bInterfaceNumber != MY_INTERFACE) {
 		ERR("Wrong Interface number %d\n", iface_desc->bInterfaceNumber);
-		return -EINVAL;
+		return 0;
 	}
 	if(iface_desc->bNumEndpoints != MY_ENDPOINTS) {
 		ERR("Wrong number of endpoints: %d\n", iface_desc->bNumEndpoints);
-		return -EINVAL;
+		return 0;
 	}
 	endpoint = iface_desc->endpoint;
+	mydev->is_usb2 = (endpoint->wMaxPacketSize == 512);
 	for(i = 0; i < iface_desc->bNumEndpoints; i++, endpoint++) {
 		if(endpoint->bEndpointAddress != my_endpoints[i]) {
 			ERR("Wrong endpoint %d: address = 0x%X\n", i, endpoint->bEndpointAddress);
-			return -EINVAL;
+			return 0;
 		}
 		if(endpoint->bEndpointAddress == MY_EP_OUT || endpoint->bEndpointAddress == MY_EP_IN) {
 			if(endpoint->wMaxPacketSize > PACKET_SIZE) {
 				ERR("Endpoint #%d wMaxPacketSize too large (%d)\n", i, endpoint->wMaxPacketSize);
-				return -EINVAL;
+				return 0;
 			}
 		}
 	}
-	if(usb_reset(handle) != 0) {
-		ERR("Reseting device: usb: %s\n", usb_strerror());
+	if(usb_resetep(mydev->handle, MY_EP_OUT) != 0) {
+		ERR("Failed to reset usb output endpoint: %s\n", usb_strerror());
+		return 0;
 	}
-	if(usb_set_configuration(handle, MY_CONFIG) != 0) {
-		ERR("usb: %s\n", usb_strerror());
-		return -EINVAL;
+	if(usb_resetep(mydev->handle, MY_EP_IN) != 0) {
+		ERR("Failed to reset usb input endpoint: %s\n", usb_strerror());
+		return 0;
 	}
-	if(usb_claim_interface(handle, MY_INTERFACE) != 0) {
-		ERR("usb: %s\n", usb_strerror());
-		return -EINVAL;
-	}
-	if(usb_resetep(handle, MY_EP_OUT) != 0) {
-		ERR("usb: %s\n", usb_strerror());
-		return -EINVAL;
-	}
-	if(usb_resetep(handle, MY_EP_IN) != 0) {
-		ERR("usb: %s\n", usb_strerror());
-		return -EINVAL;
-	}
-	ret = get_usb_string(iManufacturer, BUFSIZ, dev_desc->iManufacturer, handle);
-	ret = get_usb_string(iProduct, BUFSIZ, dev_desc->iProduct, handle);
-	if(verbose >= LOG_INFO)
-		INFO("iManufacturer=%s iProduct=%s\n", iManufacturer, iProduct);
-	return 0;
+	ret = get_usb_string(mydev->iManufacturer, BUFSIZ, dev_desc->iManufacturer, mydev->handle);
+	ret = get_usb_string(mydev->iProduct, BUFSIZ, dev_desc->iProduct, mydev->handle);
+	ret = get_usb_string(mydev->iSerialNumber, BUFSIZ, dev_desc->iSerialNumber, mydev->handle);
+	return 1;
 }
 
-int renumerate_device(struct usb_dev_handle *handle)
+int renumerate_device(struct my_usb_device *mydev)
 {
 	char				buf[PACKET_SIZE];
 	struct fpga_packet_header	*phead = (struct fpga_packet_header *)buf;
 	int				ret;
 
-	assert(handle != NULL);
-	if(verbose >= LOG_INFO)
-		INFO("Renumerating\n");
+	assert(mydev != NULL);
+	DBG("Renumerating\n");
 	phead->header.op = RENUMERATE;
-	ret = usb_bulk_write(handle, MY_EP_OUT, (char *)phead, 1, TIMEOUT);
+	ret = usb_bulk_write(mydev->handle, MY_EP_OUT, (char *)phead, 1, TIMEOUT);
 	if(ret < 0) {
-		ERR("usb: bulk_write failed (%d)\n", ret);
+		ERR("bulk_write failed: %s\n", usb_strerror());
 		return ret;
 	} else if(ret != 1) {
-		ERR("usb: bulk_write short write (%d)\n", ret);
+		ERR("bulk_write short write: %s\n", usb_strerror());
 		return -EFAULT;
 	}
 	return 0;
 }
 
-int fpga_load(struct usb_dev_handle *handle, const struct hexdata *hexdata)
+/*
+ * Returns: true on success, false on failure
+ */
+int fpga_load(struct my_usb_device *mydev, const struct hexdata *hexdata)
 {
 	unsigned int	i;
+	unsigned int	j = 0;
 	int		ret;
 	int		finished = 0;
+	const char	*v = hexdata->version_info;
 	
-	assert(handle != NULL);
-	if(verbose >= LOG_INFO)
-		INFO("Start...\n");
-		
+	v = (v[0]) ? v : "Unknown";
+	assert(mydev != NULL);
+	INFO("FPGA_LOAD (version %s)\n", v);
+	/*
+	 * i - is the line number
+	 * j - is the sequence number, on USB 2, i=j, but on
+	 *     USB 1 send_splited_hexline may increase the sequence
+	 *     number, as it needs 
+	 */
 	for(i = 0; i < hexdata->maxlines; i++) {
 		struct hexline	*hexline = hexdata->lines[i];
 
@@ -514,58 +570,24 @@ int fpga_load(struct usb_dev_handle *handle, const struct hexdata *hexdata)
 			return 0;
 		}
 		if(hexline->d.content.header.tt == TT_EOF) {
-			INFO("End of data\n");
+			DBG("End of data\n");
 			finished = 1;
 			continue;
 		}
-		if((ret = send_hexline(handle, hexline, i)) != 0) {
-			perror("Failed sending hexline");
-			return 0;
+		if(mydev->is_usb2) {
+			if((ret = send_hexline(mydev, hexline, i)) != 0) {
+				perror("Failed sending hexline");
+				return 0;
+			}
+		} else {
+			if((ret = send_splited_hexline(mydev, hexline, j, 60)) < 0) {
+				perror("Failed sending hexline (splitting did not help)");
+				return 0;
+			}
+			j += ret;
 		}
 	}
-	if(verbose >= LOG_INFO)
-		INFO("Finished...\n");
-	return 1;
-}
-
-int fpga_load_usb1(struct usb_dev_handle *handle, const struct hexdata *hexdata)
-{
-	unsigned int	i,j=0;
-	int		ret;
-	int		finished = 0;
-	
-	assert(handle != NULL);
-	if(verbose >= LOG_INFO)
-		INFO("Start...\n");
-		
-	// i - is the line number
-	// j - is the sequence number, on USB 2, i=j, but on
-	//     USB 1 send_splited_hexline may increase the sequence
-	//     number, as it needs 
-	for(i = 0; i < hexdata->maxlines; i++) {
-		struct hexline	*hexline = hexdata->lines[i];
-		
-
-		if(!hexline)
-			break;
-		if(finished) {
-			ERR("Extra data after End Of Data Record (line %d)\n", i);
-			return 0;
-		}
-		if(hexline->d.content.header.tt == TT_EOF) {
-			INFO("End of data\n");
-			finished = 1;
-			continue;
-		}
-		
-		if((ret = send_splited_hexline(handle, hexline, j, 60)) < 0) {
-			perror("Failed sending hexline (splitting did not help)");
-			return 0;
-		}
-		j += ret;
-	}
-	if(verbose >= LOG_INFO)
-		INFO("Finished...\n");
+	DBG("Finished...\n");
 	return 1;
 }
 
@@ -573,10 +595,9 @@ int fpga_load_usb1(struct usb_dev_handle *handle, const struct hexdata *hexdata)
 
 void usage()
 {
-	fprintf(stderr, "Usage: %s -D /proc/bus/usb/<bus>/<dev> [options...]\n", progname);
+	fprintf(stderr, "Usage: %s -D {/proc/bus/usb|/dev/bus/usb}/<bus>/<dev> [options...]\n", progname);
 	fprintf(stderr, "\tOptions:\n");
 	fprintf(stderr, "\t\t[-b <binfile>]	# output to <binfile>\n");
-	fprintf(stderr, "\t\t[-d]		# Get device version from eeprom\n");
 	fprintf(stderr, "\t\t[-I <hexfile>]	# Input from <hexfile>\n");
 	fprintf(stderr, "\t\t[-g]		# Get eeprom from device\n");
 #ifdef XORCOM_INTERNAL
@@ -599,27 +620,14 @@ static void parse_report_func(int level, const char *msg, ...)
 	va_end(ap);
 }
 
-int hasUSB2( struct usb_device	*dev )
-{
-	if (dev->config->interface->altsetting->endpoint->wMaxPacketSize != 512)
-		return 0;
-	else
-		return 1;
-}
-
-// usb_interface_descriptor->usb_endpoint_descriptor.wMaxPacketSize
-
 int main(int argc, char *argv[])
 {
-	struct usb_device	*dev;
-	usb_dev_handle		*handle;
+	struct my_usb_device	mydev;
 	const char		*devpath = NULL;
 	const char		*binfile = NULL;
 	const char		*hexfile = NULL;
 	struct hexdata		*hexdata = NULL;
-	struct myeeprom		eeprom_buf;
 	int			opt_read_eeprom = 0;
-	int			opt_print_bcdver_only = 0;
 #ifdef	XORCOM_INTERNAL
 	int			opt_write_eeprom = 0;
 	char			*vendor = NULL;
@@ -629,9 +637,9 @@ int main(int argc, char *argv[])
 	char			*release = NULL;
 	char			*serial = NULL;
 	uint8_t			serial_buf[SERIAL_SIZE];
-	const char		options[] = "b:C:dD:ghI:vV:P:R:S:";
+	const char		options[] = "b:C:D:ghI:vV:P:R:S:";
 #else
-	const char		options[] = "b:dD:ghI:v";
+	const char		options[] = "b:D:ghI:v";
 #endif
 	int			ret = 0;
 
@@ -652,10 +660,6 @@ int main(int argc, char *argv[])
 			case 'b':
 				binfile = optarg;
 				break;
-			case 'd':
-				opt_print_bcdver_only = 1;
-				opt_read_eeprom = 1;
-				break;
 			case 'g':
 				opt_read_eeprom = 1;
 				break;
@@ -730,97 +734,72 @@ int main(int argc, char *argv[])
 		ERR("Missing device path\n");
 		usage();
 	}
-	if(verbose)
-		INFO("Startup %s\n", devpath);
+#ifdef	XORCOM_INTERNAL
+	if(vendor || product || release || serial || source )
+		opt_read_eeprom = opt_write_eeprom = 1;
+#endif
+	DBG("Startup %s\n", devpath);
 
-	usb_init();
-	usb_find_busses();
-	usb_find_devices();
-	dev = dev_of_path(devpath);
-	if(!dev) {
-		ERR("Bailing out\n");
-		exit(1);
-	}
-	handle = usb_open(dev);
-	if(!handle) {
-		ERR("Failed to open usb device '%s/%s': %s\n", dev->bus->dirname, dev->filename, usb_strerror());
-		return -ENODEV;
-	}
-	if(my_usb_device(dev, handle)) {
-		ERR("Foreign usb device '%s/%s'\n", dev->bus->dirname, dev->filename);
+	if(!my_usb_device_init(devpath, &mydev)) {
+		ERR("Failed to initialize USB device '%s'\n", devpath);
 		ret = -ENODEV;
 		goto dev_err;
 	}
-	
+	ret = eeprom_get(&mydev);
+	if(ret < 0) {
+		ERR("Failed reading eeprom\n");
+		goto dev_err;
+	}
+	if(opt_read_eeprom) {
+		show_device_info(&mydev);
+	}
 	if(hexdata) {
-		int status;
-
-		if (hasUSB2(dev))
-			status = fpga_load(handle, hexdata);
-		else {
+		if (!mydev.is_usb2)
 			INFO("Warning: working on a low end USB1 backend\n");
-			status = fpga_load_usb1(handle, hexdata);
-		}
-
-		if(!status) {
+		if(!fpga_load(&mydev, hexdata)) {
 			ERR("FPGA loading failed\n");
 			ret = -ENODEV;
 			goto dev_err;
 		}
-		ret = renumerate_device(handle);
+		ret = renumerate_device(&mydev);
 		if(ret < 0) {
 			ERR("Renumeration failed: errno=%d\n", ret);
 			goto dev_err;
 		}
 	}
-#ifdef	XORCOM_INTERNAL
-	if(vendor || product || release || serial || source )
-		opt_read_eeprom = opt_write_eeprom = 1;
-#endif
-	if(opt_read_eeprom) {
-		ret = eeprom_get(handle, &eeprom_buf);
-		if(ret < 0) {
-			ERR("Failed reading eeprom: %d\n", ret);
-			goto dev_err;
-		}
-		if (opt_print_bcdver_only)
-			print_bcd_ver(&eeprom_buf);
-		else
-			dump_eeprom(&eeprom_buf);
-	}
 #ifdef XORCOM_INTERNAL
 	if(opt_write_eeprom) {
 		// FF: address source is from device. C0: from eeprom
 		if (is_source_given)
-			eeprom_buf.source = strtoul(source, NULL, 0);
+			mydev.eeprom.source = strtoul(source, NULL, 0);
 		else
-			eeprom_buf.source = 0xC0;
+			mydev.eeprom.source = 0xC0;
 		if(vendor)
-			eeprom_buf.vendor = strtoul(vendor, NULL, 0);
+			mydev.eeprom.vendor = strtoul(vendor, NULL, 0);
 		if(product)
-			eeprom_buf.product = strtoul(product, NULL, 0);
+			mydev.eeprom.product = strtoul(product, NULL, 0);
 		if(release) {
 			int	release_major = 0;
 			int	release_minor = 0;
 
 			sscanf(release, "%d.%d", &release_major, &release_minor);
-			eeprom_buf.release_major = release_major;
-			eeprom_buf.release_minor = release_minor;
+			mydev.eeprom.release_major = release_major;
+			mydev.eeprom.release_minor = release_minor;
 		}
 		if(serial) {
-			memcpy(eeprom_buf.serial, serial_buf, SERIAL_SIZE);
+			memcpy(&mydev.eeprom.serial, serial_buf, SERIAL_SIZE);
 		}
-		dump_eeprom(&eeprom_buf);
-		ret = eeprom_set(handle, &eeprom_buf);
+		ret = eeprom_set(&mydev, &mydev.eeprom);
 		if(ret < 0) {
-			ERR("Failed writing eeprom: %d\n", ret);
+			ERR("Failed writing eeprom: %s\n", strerror(-ret));
 			goto dev_err;
 		}
+		printf("------- RESULTS -------\n");
+		show_device_info(&mydev);
 	}
 #endif
-	if(verbose)
-		INFO("Exiting\n");
+	DBG("Exiting\n");
 dev_err:
-	usb_cleanup(handle);
+	my_usb_device_cleanup(&mydev);
 	return ret;
 }
