@@ -38,7 +38,6 @@
 #include <asm/timex.h>
 #include <linux/proc_fs.h>
 #include <linux/usb.h>
-#include <version.h>		/* For zaptel version */
 #include "xpd.h"
 #include "xproto.h"
 #include "xbus-core.h"
@@ -47,9 +46,9 @@
 #include "card_fxo.h"
 #endif
 
-static const char rcsid[] = "$Id: xpp_usb.c 1457 2006-09-09 15:24:12Z tzafrir $";
+static const char rcsid[] = "$Id: xpp_usb.c 2239 2007-02-27 06:14:18Z tzafrir $";
 
-DEF_PARM(int, print_dbg, 0, "Print DBG statements");	/* must be before zap_debug.h */
+DEF_PARM(int, print_dbg, 0, 0600, "Print DBG statements");	/* must be before zap_debug.h */
 
 #include "zap_debug.h"
 
@@ -58,7 +57,6 @@ DEF_PARM(int, print_dbg, 0, "Print DBG statements");	/* must be before zap_debug
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,14)
 #  define URB_ASYNC_UNLINK 0
 #endif
-#define	USBDEV_MAX	10
 /* Get a minor range for your devices from the usb maintainer */
 #define USB_SKEL_MINOR_BASE	192
 
@@ -79,21 +77,19 @@ struct xusb_endpoint {
 	usb_complete_t	callback;
 };
 
-static int xusb_packet_send(xbus_t *xbus, xpacket_t *pack);
+static int xusb_xframe_send(xbus_t *xbus, xframe_t *xframe);
 
 xbus_ops_t	xusb_ops = {
-	.packet_send = xusb_packet_send,
-	.packet_new = NULL,			// Default allocator
-	.packet_free = NULL,			// Default deallocator
+	.xframe_send = xusb_xframe_send,
+	.xframe_new = NULL,			// Default allocator
+	.xframe_free = NULL,			// Default deallocator
 };
 
 enum {
-	XUSB_N_RX_PACKETS,
-	XUSB_N_TX_PACKETS,
+	XUSB_N_RX_FRAMES,
+	XUSB_N_TX_FRAMES,
 	XUSB_N_RX_ERRORS,
 	XUSB_N_TX_ERRORS,
-	XUSB_N_PCM_READS,
-	XUSB_N_PCM_WRITES,
 };
 
 #define	XUSB_COUNTER(xusb, counter)	((xusb)->counters[XUSB_N_ ## counter])
@@ -103,12 +99,10 @@ enum {
 static struct xusb_counters {
 	char	*name;
 } xusb_counters[] = {
-	C_(RX_PACKETS),
-	C_(TX_PACKETS),
+	C_(RX_FRAMES),
+	C_(TX_FRAMES),
 	C_(RX_ERRORS),
 	C_(TX_ERRORS),
-	C_(PCM_READS),
-	C_(PCM_WRITES),
 };
 
 #undef C_
@@ -146,7 +140,7 @@ typedef struct xpp_usb_bus {
 } xusb_t;
 
 static	spinlock_t	xusb_lock = SPIN_LOCK_UNLOCKED;
-static xusb_t *xusb_array[USBDEV_MAX] = {};
+static xusb_t *xusb_array[MAX_BUSES] = {};
 static unsigned bus_count = 0;
 
 
@@ -164,76 +158,36 @@ static int xusb_open		(struct inode *inode, struct file *file);
 static int xusb_release		(struct inode *inode, struct file *file);
 static void xusb_write_bulk_callback	(struct urb *urb, struct pt_regs *regs);
 #endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,20)
+#define	USB_PASS_CB(u)	struct urb *u, struct pt_regs *regs
+#else
+#define	USB_PASS_CB(u)	struct urb *u
+#endif
+
 static void xpp_urb_delete(struct urb *urb);
 static struct urb *xpp_urb_new(xusb_t *dev, enum xusb_dir dir, size_t size);
-static void xpp_send_callback(struct urb *urb, struct pt_regs *regs);
-static void xpp_receive_callback(struct urb *urb, struct pt_regs *regs);
+static void xpp_send_callback(USB_PASS_CB(urb));
+static void xpp_receive_callback(USB_PASS_CB(urb));
 
 static int xusb_probe		(struct usb_interface *interface, const struct usb_device_id *id);
 static void xusb_disconnect	(struct usb_interface *interface);
 static int xusb_read_proc(char *page, char **start, off_t off, int count, int *eof, void *data);
 
 /*------------------------------------------------------------------*/
-
 #if 0
-/**
- * Allocates a new XPP packet.
- * @xbus The XPP bus in which the packet will flow (for counters 
- *       maintenance)
- * @flags Flags for kernel memory allocation.
- * @returns A pointer to the new packet, or NULL in case of failure.
- * 
- * 
- * Packet allocation/deallocation:
- * 	Sent packets:
- * 	  - Allocated by protocol commands
- * 	  - Deallocated by xmus_xmitter
- * 	Receive packets:
- * 	  - Allocated/deallocated by xbus_xmiter
- */
-xpacket_t	*xusb_packet_new(xbus_t *xbus, int flags)
-{
-	xpacket_t	*pack;
-
-	/* To avoid races we increament counter in advance and decrement it later 
-	 * in case of failure */
-	atomic_inc(&xbus->packet_counter); 
-	//DBG("Incremented packet_counter of bus %s (new packet) to %d\n", 
-	//		xbus->busname, atomic_read(&xbus->packet_counter));
-	pack = kmem_cache_alloc(packet_cache, flags);
-	if (pack) {
-		memset(pack, 0, sizeof(xpacket_t));
-		atomic_inc(&xpacket_count);
-	} else {
-		atomic_dec(&xbus->packet_counter);
-		//DBG("Decremented packet_counter of bus %s (failed new packet) to %d\n", 
-		//		xbus->busname, atomic_read(&xbus->packet_counter));
-	}
-	return pack;
-}
-
-void xusb_packet_free(xbus_t *xbus, xpacket_t *p)
-{
-	kmem_cache_free(packet_cache, p);
-	atomic_dec(&xpacket_count);
-	atomic_dec(&xbus->packet_counter);
-	//DBG("Decremented packet_counter of bus %s (freed packet) to %d\n", 
-	//		xbus->busname, atomic_read(&xbus->packet_counter));
-}
-
-#endif
-
 #ifndef DEBUG
 
 #define	packet_debug(m, x, p)
 
 #else
 
+/* FIXME FIXME FIXME: should move code to card methods */
 static void packet_debug(const char msg[], xusb_t *xusb, xpacket_t *pack)
 {
 	char	title[XBUS_DESCLEN];
 
-	if(pack->content.opcode == XPROTO_NAME(GLOBAL,PCM_READ)) {
+	if(pack->opcode == XPROTO_NAME(GLOBAL,PCM_READ)) {
 #ifdef	DEBUG_PCM_TIMING
 		/*
 		 * DEBUG: high-res timing of PCM_READ to PCM_WRITE
@@ -247,7 +201,7 @@ static void packet_debug(const char msg[], xusb_t *xusb, xpacket_t *pack)
 			dump_packet("USB RECEIVE PCM", pack, print_dbg);
 #endif
 		return;
-	} else if(pack->content.opcode == XPROTO_NAME(GLOBAL,PCM_WRITE)) {
+	} else if(pack->opcode == XPROTO_NAME(GLOBAL,PCM_WRITE)) {
 #ifdef	DEBUG_PCM_TIMING
 		/*
 		 * DEBUG: high-res timing of PCM_READ to PCM_WRITE
@@ -261,23 +215,35 @@ static void packet_debug(const char msg[], xusb_t *xusb, xpacket_t *pack)
 			dump_packet("USB SEND PCM", pack, print_dbg);
 #endif
 		return;
-	} else if(pack->content.opcode == XPROTO_NAME(FXS, SLIC_WRITE)) {
-		slic_cmd_t	*sc;
+	} else if(pack->opcode == XPROTO_NAME(GLOBAL, REGISTER_REQUEST)) {
+		reg_cmd_t	*regcmd;
 
-		sc = &RPACKET_FIELD(pack, FXS, SLIC_WRITE, slic_cmd);
-		if(sc->bytes == 2 && sc->content.direct.reg_num == 0x06 && sc->content.direct.read)	/* ignore SLIC_QUERY */
+		regcmd = &RPACKET_FIELD(pack, GLOBAL, REGISTER_REQUEST, reg_cmd);
+		if(REG_FIELD(regcmd, regnum) == 0x06)		/* ignore SLIC_QUERY */
 			return;
-		if(sc->bytes == 2 && sc->content.direct.reg_num == DAA_VBAT_REGISTER && sc->content.direct.read)	/* ignore DAA_QUERY */
+		if(REG_FIELD(regcmd, regnum) == DAA_VBAT_REGISTER)	/* ignore DAA_QUERY */
 			return;
-	} else if(pack->content.opcode == XPROTO_NAME(FXS, SLIC_REPLY)) {
+		if(REG_FIELD(regcmd, regnum) == 0x30) {		/* ignore BRI query */
+#if 0
+			static	int rate_limit;
+			if((rate_limit++ % 1000) < 10)
+				dump_packet("BRI STATE REG", pack, print_dbg);
+#endif
 			return;
+		}
+#if 0
+	} else if(pack->opcode == XPROTO_NAME(FXS, REGISTER_REPLY)) {
+			return;
+#endif
 	}
 	snprintf(title, XBUS_DESCLEN, "%s: %s", msg, xusb->xbus->busname);
 	dump_packet(title, pack, print_dbg);
 }
+
+#endif
 #endif
 
-static int xusb_packet_send(xbus_t *xbus, xpacket_t *pack)
+static int xusb_xframe_send(xbus_t *xbus, xframe_t *xframe)
 {
 	xusb_t			*xusb;
 	struct urb		*urb;
@@ -285,7 +251,7 @@ static int xusb_packet_send(xbus_t *xbus, xpacket_t *pack)
 	size_t			size;
 	struct xusb_endpoint	*xusb_ep;
 
-	BUG_ON(!pack);
+	BUG_ON(!xframe);
 	BUG_ON(!xbus);
 	xusb = xbus->priv;
 	BUG_ON(!xusb);
@@ -300,12 +266,15 @@ static int xusb_packet_send(xbus_t *xbus, xpacket_t *pack)
 	if(atomic_read(&xusb->pending_writes) > MAX_PENDING_WRITES) {
 		static	int rate_limit;
 
-		if((rate_limit++ % 1000) < 10)
-			ERR("%s: %s: more than %d pending writes. Dropping.\n", __FUNCTION__, xbus->busname, MAX_PENDING_WRITES);
+		if((rate_limit++ % 5000) == 0)
+			ERR("%s: %s: more than %d pending writes (%d). Dropping.\n",
+					__FUNCTION__, xbus->busname, MAX_PENDING_WRITES, rate_limit);
 		ret = -ENODEV;
 		goto freepack;
 	}
-	size = PACKET_LEN(pack);
+	if (print_dbg)
+		dump_xframe("USB_FRAME_SEND", xbus, xframe);
+	size = XFRAME_LEN(xframe);
 	xusb_ep = &xusb->endpoints[XUSB_SEND];
 	urb = xpp_urb_new(xusb, XUSB_SEND, size);
 	if (!urb) {
@@ -313,10 +282,10 @@ static int xusb_packet_send(xbus_t *xbus, xpacket_t *pack)
 		ret = -ENOMEM;
 		goto freepack;
 	}
-	packet_debug("USB_PACKET_SEND", xusb, pack);
 
 	/* FIXME: FIXME: FIXME: we use copy+free until low-level drivers allocate memory themselves */
-	memcpy(urb->transfer_buffer, &pack->content, size);
+	
+	memcpy(urb->transfer_buffer, xframe->packets, size);
 
 	ret = usb_submit_urb(urb, GFP_ATOMIC);
 	if(ret < 0) {
@@ -329,10 +298,8 @@ static int xusb_packet_send(xbus_t *xbus, xpacket_t *pack)
 		goto freepack;
 	}
 	atomic_inc(&xusb->pending_writes);
-	if(pack->content.opcode == XPROTO_NAME(GLOBAL,PCM_WRITE))
-		XUSB_COUNTER(xusb, PCM_WRITES)++;
 freepack:
-	xbus->ops->packet_free(xbus, pack);	// FIXME: eventually will be done in the urb callback
+	xbus->ops->xframe_free(xbus, xframe);	// FIXME: eventually will be done in the urb callback
 	if(ret < 0)
 		XUSB_COUNTER(xusb, TX_ERRORS)++;
 	return ret;
@@ -405,16 +372,16 @@ static const struct xusb_model_info {
 	struct xusb_endpoint	out;
 	xbus_type_t		bus_type;
 } model_table[] = {
-	XUSB_MODEL(0x86, 0x02, FIRMWARE_LOOPBACK,	"bulkloop.hex"),
-	XUSB_MODEL(0x86, 0x02, FIRMWARE_LOOPBACK,	"FPGA_bulkloop.hex"),
-	XUSB_MODEL(0x86, 0x02, FIRMWARE_XPP,		"FPGA_XPD.hex"),
+	XUSB_MODEL(0x86, 0x02, FIRMWARE_XPP,		"FPGA_XPD"),
 };
 
 /* table of devices that work with this driver */
 static const struct usb_device_id xusb_table [] = {
 //	{ USB_DEVICE(0x04B4, 0x8613) }, // default of cypress
-	{ USB_DEVICE(0xE4E4, 0x2211), .driver_info=(kernel_ulong_t)&model_table[2] }, // FPGA_XPD.hex
-	{ USB_DEVICE(0xE4E4, 0x1132), .driver_info=(kernel_ulong_t)&model_table[2] }, // FPGA_XPD.hex
+	{ USB_DEVICE(0xE4E4, 0x2211), .driver_info=(kernel_ulong_t)&model_table[0] }, // OLD FPGA
+	{ USB_DEVICE(0xE4E4, 0x1132), .driver_info=(kernel_ulong_t)&model_table[0] }, // FPGA_FXS
+	{ USB_DEVICE(0xE4E4, 0x1142), .driver_info=(kernel_ulong_t)&model_table[0] }, // FPGA_1141
+	{ USB_DEVICE(0xE4E4, 0x1152), .driver_info=(kernel_ulong_t)&model_table[0] }, // FPGA_1151
 	/* "Gadget Zero" firmware runs under Linux */
 	//{ USB_DEVICE(0x0525, 0xa4a0) },
 	{ }					/* Terminating entry */
@@ -499,7 +466,7 @@ static int set_endpoints(xusb_t *xusb, struct usb_interface *interface, struct x
 		}
 		if(usb_pipein(ep_addr)) {	// Input
 			if(ep_addr == model_info->in.ep_addr) {
-				if(endpoint->wMaxPacketSize < sizeof(xpacket_raw_t)) {
+				if(endpoint->wMaxPacketSize < sizeof(xframe_t)) {
 					NOTICE("USB input endpoint 0x%X support only wMaxPacketSize=%d (need USB-2)\n", ep_addr, endpoint->wMaxPacketSize);
 				}
 				xusb_ep = &xusb->endpoints[XUSB_RECV];
@@ -509,7 +476,7 @@ static int set_endpoints(xusb_t *xusb, struct usb_interface *interface, struct x
 			}
 		} else {			// Output
 			if(ep_addr == model_info->out.ep_addr) {
-				if(endpoint->wMaxPacketSize < sizeof(xpacket_raw_t)) {
+				if(endpoint->wMaxPacketSize < sizeof(xframe_t)) {
 					NOTICE("USB output endpoint 0x%X support only wMaxPacketSize=%d (need USB-2)\n", ep_addr, endpoint->wMaxPacketSize);
 				}
 				xusb_ep = &xusb->endpoints[XUSB_SEND];
@@ -547,7 +514,14 @@ static int xusb_probe(struct usb_interface *interface, const struct usb_device_i
 	
 	INFO("New XUSB device MODEL=%s bus_type=%d\n", model_info->desc, model_info->bus_type);
 
-	if((retval = usb_reset_device(udev)) < 0) {
+	/* The USB stack before 2.6.10 seems to be a bit shoddy. It seems that when being called
+	 * from the probe we may already have the lock to udev (the Usb DEVice). Thus we call
+	 * the internal __usb_reset_device instead. */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,10)
+	if((retval = __usb_reset_device(udev)) < 0) {
+#else
+	if((retval =   usb_reset_device(udev)) < 0) {
+#endif
 		ERR("usb_reset_device failed: %d\n", retval);
 		goto probe_failed;
 	}
@@ -612,12 +586,12 @@ static int xusb_probe(struct usb_interface *interface, const struct usb_device_i
 	xbus->max_packet_size = min(xusb->endpoints[XUSB_SEND].max_size , xusb->endpoints[XUSB_RECV].max_size);
 
 	spin_lock_irqsave(&xusb_lock, flags);
-	for(i = 0; i < USBDEV_MAX; i++) {
+	for(i = 0; i < MAX_BUSES; i++) {
 		if(xusb_array[i] == NULL)
 			break;
 	}
 	spin_unlock_irqrestore(&xusb_lock, flags);
-	if(i >= USBDEV_MAX) {
+	if(i >= MAX_BUSES) {
 		ERR("xpp_usb: Too many XPP USB buses\n");
 		retval = -ENOMEM;
 		goto probe_failed;
@@ -703,11 +677,11 @@ static void xusb_disconnect(struct usb_interface *interface)
 	xbus = xusb->xbus;
 
 	/* find our xusb */
-	for(i = 0; i < USBDEV_MAX; i++) {
+	for(i = 0; i < MAX_BUSES; i++) {
 		if(xusb_array[i] == xusb)
 			break;
 	}
-	BUG_ON(i >= USBDEV_MAX);
+	BUG_ON(i >= MAX_BUSES);
 	xusb_array[i] = NULL;
 
 #ifdef CONFIG_PROC_FS
@@ -742,7 +716,7 @@ static void xusb_disconnect(struct usb_interface *interface)
 	INFO("XUSB #%d now disconnected\n", minor);
 }
 
-static void xpp_send_callback(struct urb *urb, struct pt_regs *regs)
+static void xpp_send_callback(USB_PASS_CB(urb))
 {
 	xusb_t			*xusb = (xusb_t *)urb->context;
 	xbus_t			*xbus = xusb->xbus;
@@ -762,14 +736,15 @@ static void xpp_send_callback(struct urb *urb, struct pt_regs *regs)
 		return;
 	}
 	/* allow device read, write and ioctl */
-	XUSB_COUNTER(xusb, TX_PACKETS)++;
+	XUSB_COUNTER(xusb, TX_FRAMES)++;
 }
 
-static void xpp_receive_callback(struct urb *urb, struct pt_regs *regs)
+static void xpp_receive_callback(USB_PASS_CB(urb))
 {
 	xusb_t		*xusb = (xusb_t *)urb->context;
 	xbus_t		*xbus;
 	xpacket_t	*pack;
+	xframe_t	*xframe;
 	size_t		size;
 	int		retval;
 	bool		do_resubmit = 1;
@@ -805,25 +780,22 @@ static void xpp_receive_callback(struct urb *urb, struct pt_regs *regs)
 		do_resubmit = 0;
 		goto end;
 	}
-	pack = xbus->ops->packet_new(xbus, GFP_ATOMIC);
-	if(!pack) {
+	xframe = xbus->ops->xframe_new(xbus, GFP_ATOMIC);
+	if(!xframe) {
 		ERR("%s: Not enough memory for packets. Dropping\n", __FUNCTION__);
 		goto end;
 	}
-
 	size = urb->actual_length;
-	memcpy(&pack->content, urb->transfer_buffer, size);
+	atomic_set(&xframe->frame_len, size);
+	pack = (xpacket_t *)xframe->packets;
+	memcpy(xframe->packets, urb->transfer_buffer, size);
 
-	pack->datalen = size - sizeof(xpd_addr_t) - 1; 	// opcode size
-	// DBG("datalen of new packet: %d\n", pack->datalen);
-
-	packet_debug("USB_PACKET_RECEIVE", xusb, pack);
-	XUSB_COUNTER(xusb, RX_PACKETS)++;
-	if(pack->content.opcode == XPROTO_NAME(GLOBAL,PCM_READ))
-		XUSB_COUNTER(xusb, PCM_READS)++;
+	if (print_dbg)
+		dump_xframe("USB_FRAME_RECEIVE", xbus, xframe);
+	XUSB_COUNTER(xusb, RX_FRAMES)++;
 	// Send UP
-	packet_receive(xbus, pack);
-end:	
+	xframe_receive(xbus, xframe);
+end:
 	if(is_inuse)
 		up_read(&xbus->in_use);
 	if(do_resubmit) {
@@ -843,7 +815,7 @@ int __init xpp_usb_init(void)
 	int result;
 	//xusb_t *xusb;
 
-	INFO("%s revision %s\n", THIS_MODULE->name, ZAPTEL_VERSION);
+	INFO("%s revision %s\n", THIS_MODULE->name, XPP_VERSION);
 
 	/* register this driver with the USB subsystem */
 	result = usb_register(&xusb_driver);
@@ -924,7 +896,7 @@ out:
 MODULE_DESCRIPTION("XPP USB Driver");
 MODULE_AUTHOR("Oron Peled <oron@actcom.co.il>");
 MODULE_LICENSE("GPL");
-MODULE_VERSION(ZAPTEL_VERSION);
+MODULE_VERSION(XPP_VERSION);
 
 module_init(xpp_usb_init);
 module_exit(xpp_usb_cleanup);
