@@ -28,10 +28,10 @@
 #include <linux/kernel.h>
 #include <linux/errno.h>
 #include <linux/module.h>
-#include "zaptel.h"
+#include <zaptel.h>
 #include "zap_debug.h"
 
-static const char rcsid[] = "$Id: zap_debug.c 1212 2006-07-08 00:43:31Z tzafrir $";
+static const char rcsid[] = "$Id: zap_debug.c 1966 2007-01-25 10:48:33Z tzafrir $";
 
 #define	P_(x)	[ x ] = { .value = x, .name = #x, }
 static	struct {
