@@ -1,7 +1,7 @@
 /*
  * Configuration program for Zapata Telephony Interface
  *
- * Written by Mark Spencer <markster@linux-support.net>
+ * Written by Mark Spencer <markster@digium.com>
  * Based on previous works, designs, and architectures conceived and
  * written by Jim Dixon <jim@lambdatel.com>.
  *
@@ -24,7 +24,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
  *
- * Primary Author: Mark Spencer <markster@linux-support.net>
+ * Primary Author: Mark Spencer <markster@digium.com>
  * Radio Support by Jim Dixon <jim@lambdatel.com>
  */
 
@@ -1281,6 +1281,16 @@ int main(int argc, char *argv[])
 							current_state.channo = cc[x].chan | ZT_GET_PARAMS_RETURN_MASTER;
 							if (ioctl(fd, ZT_GET_PARAMS, &current_state))
 								needupdate = 1;
+	 						else if (current_state.chanpos == 16 &&
+	 							(cc[x].sigtype & ZT_SIG_CLEAR) && 
+	 							!(lc[current_state.spanno - 1].lineconfig &
+	 							(ZT_CONFIG_D4|ZT_CONFIG_ESF|ZT_CONFIG_B8ZS|ZT_CONFIG_CCS))) {
+	 								fprintf(stderr, 
+	 									"CAS signalling on span %u conflicts with %s on channel %u.\n",
+	 									current_state.spanno, 
+	 									sigtype_to_str(cc[x].sigtype), x);
+	 								exit(1);
+	 						}
 						}
 
 						if (!needupdate) {
