@@ -1,7 +1,7 @@
 /*
  * Dynamic Span Interface for Zaptel
  *
- * Written by Mark Spencer <markster@linux-support.net>
+ * Written by Mark Spencer <markster@digium.com>
  *
  * Copyright (C) 2001, Linux Support Services, Inc.
  *
@@ -829,7 +829,7 @@ module_param(debug, int, 0600);
 MODULE_PARM(debug, "i");
 #endif
 MODULE_DESCRIPTION("Zaptel Dynamic Span Support");
-MODULE_AUTHOR("Mark Spencer <markster@linux-support.net>");
+MODULE_AUTHOR("Mark Spencer <markster@digium.com>");
 #ifdef MODULE_LICENSE
 MODULE_LICENSE("GPL");
 #endif
